--- asterisk-1.4.14/apps/app_meetme.c~	2007-11-15 11:19:28.000000000 -0600
+++ asterisk-1.4.14/apps/app_meetme.c	2007-11-29 12:41:06.000000000 -0600
@@ -360,6 +360,7 @@
 	struct ast_channel *chan;               /*!< Connected channel */
 	int talking;                            /*!< Is user talking */
 	int zapchannel;                         /*!< Is a Zaptel channel */
+	int dtmf_queued;			/*!< Are we currently sending DTMF? */
 	char usrvalue[50];                      /*!< Custom User Value */
 	char namerecloc[PATH_MAX];				/*!< Name Recorded file Location */
 	time_t jointime;                        /*!< Time the user joined the conference */
@@ -1269,13 +1270,44 @@
 	const struct ast_conf_user *sender, struct ast_frame *f)
 {
 	struct ast_conf_user *user;
+	int faked_dtmf_begin = 0;
 
 	AST_LIST_TRAVERSE(&conf->userlist, user, list) {
 		if (user == sender)
 			continue;
+		if (f->frametype == AST_FRAME_DTMF_BEGIN) {
+			user->dtmf_queued = 1;
+		} else if (f->frametype == AST_FRAME_DTMF_END) {
+			if (user->dtmf_queued) {
+				user->dtmf_queued = 0;
+			} else {
+				/* Send a DTMF_BEGIN since we haven't already. */
+				faked_dtmf_begin = 1;
+				user->dtmf_queued = 1;
+				f->frametype = AST_FRAME_DTMF_BEGIN;
+				if (ast_write(user->chan, f) < 0)
+					ast_log(LOG_WARNING, "Error writing frame to channel %s\n", user->chan->name);
+				f->frametype = AST_FRAME_DTMF_END;
+				continue;
+			}
+		}
 		if (ast_write(user->chan, f) < 0)
 			ast_log(LOG_WARNING, "Error writing frame to channel %s\n", user->chan->name);
 	}
+
+	/* Turn off all the DTMF generators we just turned on. */
+	if (faked_dtmf_begin) {
+		usleep(250 * 1000);
+		AST_LIST_TRAVERSE(&conf->userlist, user, list) {
+			if (user == sender)
+				continue;
+			if (user->dtmf_queued) {
+				user->dtmf_queued = 0;
+				if (ast_write(user->chan, f) < 0)
+					ast_log(LOG_WARNING, "Error writing frame to channel %s\n", user->chan->name);
+			}
+		}
+	}
 }
 
 static void sla_queue_event_full(enum sla_event_type type, 
@@ -1356,6 +1388,8 @@
 		/* Take the conference room number out of an inuse state */
 		if ((sscanf(conf->confno, "%d", &confno_int) == 1) && (confno_int >= 0 && confno_int < 1024))
 			conf_map[confno_int] = 0;
+		if (conf->fd >= 0)
+			close(conf->fd);
 		conf_free(conf);
 		res = 1;
 	}
@@ -1907,7 +1941,9 @@
 					f = ast_read(c);
 				if (!f)
 					break;
-				if ((f->frametype == AST_FRAME_VOICE) && (f->subclass == AST_FORMAT_SLINEAR)) {
+				if (f->frametype == AST_FRAME_NULL) {
+					/* Ignore NULL frames. It is perfectly normal to get these if the person is muted. */
+				} else if ((f->frametype == AST_FRAME_VOICE) && (f->subclass == AST_FORMAT_SLINEAR)) {
 					if (user->talk.actual)
 						ast_frame_adjust_volume(f, user->talk.actual);
 
@@ -2136,7 +2172,7 @@
 
 					conf_flush(fd, chan);
 				} else if ((f->frametype == AST_FRAME_DTMF_BEGIN || f->frametype == AST_FRAME_DTMF_END)
-					&& confflags & CONFFLAG_PASS_DTMF) {
+					   && (confflags & CONFFLAG_PASS_DTMF)) {
 					conf_queue_dtmf(conf, user, f);
 				} else if ((confflags & CONFFLAG_SLA_STATION) && f->frametype == AST_FRAME_CONTROL) {
 					switch (f->subclass) {
@@ -2146,8 +2182,6 @@
 					default:
 						break;
 					}
-				} else if (f->frametype == AST_FRAME_NULL) {
-					/* Ignore NULL frames. It is perfectly normal to get these if the person is muted. */
 				} else if (option_debug) {
 					ast_log(LOG_DEBUG,
 						"Got unrecognized frame on channel %s, f->frametype=%d,f->subclass=%d\n",
